;; -*- mode: scheme; coding: utf-8 -*-

;;;; Copyright (C) 2011, 2012
;;;; Free Software Foundation, Inc.
;;;;
;;;; This library is free software; you can redistribute it and/or
;;;; modify it under the terms of the GNU General Public License as
;;;; published by the Free Software Foundation; either version 3 of
;;;; the License, or (at your option) any later version.
;;;; 
;;;; This library is distributed in the hope that it will be useful,
;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;;; General Public License for more details.
;;;; 
;;;; You should have received a copy of the GNU General Public License
;;;; along with this library; if not, write to the Free Software
;;;; Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
;;;; 02110-1301 USA
;;;;

;;; Commentary:

;;; Code:

(define-module (system xft)
  ;; guile
  :use-module (ice-9 receive)
  :use-module (ice-9 rdelim)
  :use-module (ice-9 popen)
  ;; common
  :use-module (system aglobs)
  :use-module (strings strings)
  :use-module (nbs all)
  :use-module (macros push)
  :use-module (macros reexport)

  :export (xft/get-settings
	   xft/get))

(define xft/get-settings #f)
(define xft/get #f)

(eval-when (compile load eval)
  (re-export-public-interface (system aglobs)
			      (strings strings)
			      (macros push))

  (define (get-xft-settings)
    (let ((results (list))
	  (s (open-input-pipe "xrdb -query | grep Xft")))
      (do ((line (read-line s) (read-line s)))
	  ((eof-object? line))
	(set! results (cons line results)))
      (close s)
      (reverse! results)))

  (let ((xft-settings (get-xft-settings))
	(xft-settings-2 (list))
	(xft-dpi-ratio -1))
    (for-each (lambda (xft-setting)
		(let* ((tokens (str/get-tokens xft-setting (char-set #\tab)))
		       (key (car tokens))
		       (value (cadr tokens)))
		  (let* ((name (substring key 0 (1- (string-length key))))
			 (tokens (str/get-tokens name (char-set #\.)))
			 (class (car tokens))
			 (param (cadr tokens))
			 (value-2 (or (string->number value) value)))
		    (aglobs/set (string->symbol name) value-2)
		    (push! (cons (string->symbol param) value-2) xft-settings-2))))
	(get-xft-settings))
    (if (aglobs/get 'Xft.dpi)
	(aglobs/set 'Xft.dpi.ratio (/ (aglobs/get 'Xft.dpi) 96))
	(aglobs/set 'Xft.dpi.ratio 1.0))
    (aglobs/set 'apply-dpi-ratio? (not (fp/=? (aglobs/get 'Xft.dpi.ratio) 1.0 1.0E-2)))
    (push! (cons 'dpi.ratio (aglobs/get 'Xft.dpi.ratio)) xft-settings-2)
    (set! xft-settings-2 (reverse! xft-settings-2))
    (set! xft/get-settings (lambda () xft-settings-2))))


#!

(use-modules (system xft))
(reload-module (resolve-module '(system xft)))
,m (system xft)

(xft/get-settings)
(aglobs/get 'Xft.antialias)
(aglobs/get 'Xft.autohint)
(aglobs/get 'Xft.dpi)
(aglobs/get 'Xft.dpi.ratio)
(aglobs/get 'Xft.hinting)
(aglobs/get 'Xft.hintstyle)
(aglobs/get 'Xft.rgba)

(aglobs/get 'apply-dpi-ratio?)

!#
